package Mentat::Processor::Engine::Enricher;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Processor::Engine::Enricher - Engine performing ALERT message enrichment

=head1 SYNOPSIS

 use Mentat::Processor::Engine::Enricher;

=head1 DESCRIPTION

Extensive module description

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Log::Loger;
use Mentat::Core qw(:c_general);
use Mentat::Processor::Engine;
use Mentat::Message::Factory;
use Whois::Service;
use Whois::Mongo;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Mentat::Processor::Engine');
}

#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item eh_process($$) [PUBLIC]

  Usage     : Emit/execute event 'message_hello' with appropriate arguments
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object instance or given invalid arguments

=cut

sub eh_process($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($message,) = @_;
    confess "Message must be a 'Mentat::Processor::Message' implementation" unless blessed($message) and $message->isa('Mentat::Processor::Message');

    # Retrieve the encapsulated IDEA message
    my $queue_id      = $message->queue_id;
    my $queue_file    = $message->queue_file;
    my $alert_message = $message->alert_obj;

    eval {
        # Do not trigger signal hook
        local $SIG{__DIE__}; # See the eval documentation for explanation of this hack

        my $ips = $alert_message->path_values("Source/IP4");
        my %keys = ();
        foreach my $ip (@$ips) {
            my $abuse = $self->{WHOIS}->find_abuses($ip);
            if ($abuse) {
                foreach $a (split(/,/, $abuse)) {
                    $keys{$a}++;
                }
            }
        }

        my @abuses = keys(%keys);
        if (@abuses) {
            $alert_message->path_node_set('_CESNET/ResolvedAbuses', [@abuses]);

            # Replace original message with enriched version
            unlink($queue_file);
            $alert_message->to_file($queue_file);

            $self->statistics->{'cnt_message_enriched'}++;
            Log::Loger->info(__PACKAGE__, "[ENG:Enricher] Enriched message '$queue_id'");
        }

        # Send response - SUCCCESS
        $self->emit_event('message_commit', $message);
    };
    if ($@) {
        $self->statistics->{'cnt_errors_storage_other'}++;
        Log::Loger->error(__PACKAGE__, "[ENG:Enricher] Enricher error: '$@'");

        # Send response - FAILURE
        $self->emit_event('message_banish', $message);
        return EC_STOP;
    }

    return (EC_CONTINUE, $message);
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Processor::Engine instance
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my %config = @_;

    # Call parent version first
    $self = $self->SUPER::_init(%config);

    die "[ENG:Enricher] Constructor expects configurations as argument\n" unless %config;
    die "[ENG:Enricher] Enricher engine configuration 'whois_database_name' is missing\n" unless $config{whois_database_name};
    die "[ENG:Enricher] Enricher engine configuration 'whois_collection_name' is missing\n" unless $config{whois_collection_name};

    Log::Loger->info(__PACKAGE__, "[ENG:Enricher] Using storage database: '$config{whois_database_name}'");
    Log::Loger->info(__PACKAGE__, "[ENG:Enricher] Using storage collection: '$config{whois_collection_name}'");

    $self->{WHOIS} = Whois::Service->new(
            Whois::Mongo->new($config{whois_database_name}, $config{whois_collection_name}, 300),
            #Whois::Negistry->new($config{whois_dump_file}, 1, $config{whois_exceptions}),
    );

    return $self;
}

=pod

=back

=cut

1;
