package Mentat::Reporter;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Reporter

=head1 SYNOPSIS

 use Mentat::Reporter;

=head1 DESCRIPTION



=head1 CONFIGURATION OPTIONS



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1), Config::JSON(3).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use Time::HiRes qw(time);

#use Smart::Comments; #-+-> DEVEL ONLY <-+-#
use Data::Dumper; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#

# Modules from Mentat package
use Config::JSON;
use Log::Loger;
use Value::Convertor;
use Mentat::Core qw(:c_general :c_hawat);
use Mentat::Reporter::Engine::Default;
use Mentat::Reporter::Cache::Default;
use Mentat::Storage::Mongo;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#
use constant DFLT_DATABASE         => 'mentat';
use constant DFLT_LOG_COLLECTION   => 'runlogs_reporter_ng';
use constant DFLT_CACHE_COLLECTION => 'cache_threshold';

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION   = '0.1';
    $DEVEL     = 0;
    $ENV{PATH} = '/bin:/usr/bin';
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item load() [PUBLIC,STATIC]

  Usage     : my $instance = Mentat::Reporter->load($config_file);
  Purpose   : Create and return reference to the new instance with configuration taken from external JSON file
  Arguments : STRING $config_file - name of the configuration JSON file
  Returns   : Mentat::Reporter reference on success, dies on failure
  Throws    : Dies, if not invoked on class
  Comments  : Internally uses new() method
  See Also  : new() method

=cut

sub load
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    my ($config_file,) = @_;
    die "Reporter configuration file must be given as argument\n" unless $config_file;
    die "Reporter configuration file must be existing ordinary file\n" unless -f $config_file;

    # Load configurations from external JSON file
    my ($config, $error) = Config::JSON->load($config_file);
    die "$error\n" if $error;

    # Instantinate the handyman object
    return $class->new(%$config);
}

=item new() [PUBLIC,STATIC]

  Usage     : my $instance = Mentat::Reporter->new(%config);
  Purpose   : Create and return reference to the new instance
  Arguments : HASH %config with follwong keys:
  Returns   : Mentat::Reporter reference on success, dies on failure
  Throws    : Dies, if not invoked on class

=cut

sub new
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    ### Creating Mentat::Reporter instance...
    my $self = bless ({}, $class);
    return $self->_init(@_);
}

=item report() [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object

=cut

sub report(%)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    return $self->{REPORTER}->process(@_);
}

=item report_legacy() [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object

=cut

sub report_legacy(%)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    return $self->{REPORTER}->process_legacy(@_);
}

=item overview() [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object

=cut

sub overview()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

}

=item compare(%) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object

=cut

sub compare(%)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my %config = @_;
    confess "Comparison lower time boundary must be given under configration key 'start'" unless $config{start};
    confess "Comparison lower time boundary must be given under configration key 'stop'"  unless $config{stop};

    # Prior to any comparison make sure that the cache is clean
    #$self->{CACHE}->cleanup();

    # Convert the desired time boundaries to timestamps
    my $tl = Value::Convertor->datestr_to_unixs($config{start});
    my $th = Value::Convertor->datestr_to_unixs($config{stop});

    # Prepare the storage for result data
    my $analysis = {
        'thr_start_s' => $config{start},
        'thr_stop_s'  => $config{stop},
        'thr_start'   => $tl,
        'thr_stop'    => $th,
        'ts_begin'    => time,
        'ts_done'     => $tl,
        'ts_done_s'   => Value::Convertor->unixs_to_ldatestr($tl),
        'pct_done'    => 0,
    };
    # Storages for generated reports
    my $reports_legacy = [];
    my $reports_ng     = [];
    my ($r1, $r2);

    my $period1 = $self->{REPORTER}->time_period('2hrs');    # Legacy reporting period
    my $t1      = $tl;                                       # Current legacy reference time
    my $rl1     = {};                                        # Runlog for legacy processing
    my $lrl1    = {'last_ts_legacy' => ($tl - 60)};          # Last runlog for legacy processing
    my $period2 = $self->{REPORTER}->time_period('10mins');  # NG reporting period
    my $t2      = $tl;                                       # Current NG reference time
    my $rl2     = {};                                        # Runlog for NG processing
    my $lrl2    = {'last_ts_low' => ($tl - 60), 'last_ts_medium' => ($tl - 60), 'last_ts_high' => ($tl - 60), 'last_ts_critical' => ($tl - 60)};

    # Perform the comparison
    Log::Loger->info(__PACKAGE__, "[ENG:Reporter] Performing reporting algorithm comparison for time period '".(scalar localtime($tl))." - ".(scalar localtime($th))." = '".$tl."' - '".$th."')");
    my $roundbegin;
    while($t1 <= $th) {
        $roundbegin = time;

        # Perform the legacy processing
        $lrl1 = $self->_set_current_state($lrl1, $rl1);
        Log::Loger->info(__PACKAGE__, "[ENG:Reporter] ****** Simulating LEGACY reporting for base time '".(scalar localtime($t1))." = '".$t1."')");
        ($rl1, $r1) = $self->report_legacy(time => $t1, last_run_log => $lrl1);
        # Store and analyze the reports, if any
        if ($r1 and @$r1) {
            push(@$reports_legacy, @$r1);
            foreach my $r (@$r1) {
                $analysis->{'legacy'}->{'report_all_cnt'}++;
                $analysis->{'legacy'}->{'types'}->{$r->{'type'}}++;
                $analysis->{'legacy'}->{'abuses'}->{$r->{'abuse'}}++;
            }
        }

        # Perform the legacy processing
        while ($t2 <= $t1) {
            $lrl2 = $self->_set_current_state($lrl2, $rl2);
            Log::Loger->info(__PACKAGE__, "[ENG:Reporter] ***** Simulating NG reporting for base time '".(scalar localtime($t2))." = '".$t2."')");
            ($rl2, $r2) = $self->report(time => $t2, last_run_log => $lrl2);
            # Store and analyze the reports, if any
            if ($r2 and @$r2) {
                push(@$reports_ng, @$r2);
                foreach my $r (@$r2) {
                    $analysis->{'ng'}->{'report_all_cnt'}++;
                    $analysis->{'ng'}->{'types'}->{$r->{'type'}}++;
                    $analysis->{'ng'}->{'severities'}->{$r->{'severity'}}++;
                    $analysis->{'ng'}->{'abuses'}->{$r->{'abuse'}}++;
                }
            }
            $t2 += $period2;
        }

        # Update intermediate analysis
        $analysis->{'ts_done'}   = $t1;
        $analysis->{'ts_done_s'} = Value::Convertor->unixs_to_ldatestr($t1);
        $analysis->{'pct_done'}  = (($t1-$tl)/($th-$tl))*100;
        $analysis->{'round_duration'}   = time - $roundbegin;
        $analysis->{'round_duration_s'} = Value::Convertor->duration_to_string($analysis->{'round_duration'});
        $analysis->{'rounds'}++;
        $analysis->{'duration'} = (time - $analysis->{'ts_begin'});
        $analysis->{'duration_s'} = Value::Convertor->duration_to_string($analysis->{'duration'});
        if ($analysis->{'pct_done'} > 0) {
            $analysis->{'estimated'}   = ((100/$analysis->{'pct_done'}) * $analysis->{'duration'});
            $analysis->{'estimated_s'} = Value::Convertor->duration_to_string($analysis->{'estimated'});
            $analysis->{'remaining'}   = ($analysis->{'estimated'} - $analysis->{'duration'});
            $analysis->{'remaining_s'} = Value::Convertor->duration_to_string($analysis->{'remaining'});
        }
        Log::Loger->notice(__PACKAGE__, "[ENG:Reporter] ***** Intermediate results: ".Dumper($analysis));

        $t1 += $period1;
    }

    # Update analysis results
    $analysis->{'ts_end'}     = time;
    $analysis->{'duration'}   = ($analysis->{'ts_end'} - $analysis->{'ts_begin'});
    $analysis->{'duration_s'} = Value::Convertor->duration_to_string($analysis->{'duration'});
    Log::Loger->notice(__PACKAGE__, "[ENG:Reporter] ***** Final results: ".Dumper($analysis));

    # Cleanup the cache
    $self->{CACHE}->cleanup();

    return ($analysis, $reports_legacy, $reports_ng);
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Reporter instance
#  Arguments : None
#  Returns   : Mentat::Reporter reference
#  Throws    : Dies, if not invoked on instance
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my %config = @_;

    Log::Loger->notice(__PACKAGE__, "[Reporter] Config: ".Dumper(\%config));

    $self->{CACHE} = Mentat::Reporter::Cache::Default->new(%config);

    # Initialize the reporter engine
    $self->{REPORTER} = Mentat::Reporter::Engine::Default->new(%config, cache => $self->{CACHE});

    return $self;
}

sub _set_current_state($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($current_state, $runlog) = @_;

    # Determine the state of the current run
    if ($runlog and $runlog->{severities} and ref $runlog->{severities} eq 'HASH') {
        my $lth;
        foreach my $sev (keys %{$runlog->{severities}}) {
            $lth = $runlog->{severities}->{$sev}->{ts_to};
            if ($lth) {
                Log::Loger->info(__PACKAGE__, "[ENG:Reporter] * Updating current state for '$sev':'$lth'");
                $current_state->{'last_ts_'.lc($sev)} = $lth;
            }
        }
        $lth = $runlog->{legacy}->{ts_to};
        if ($lth) {
            Log::Loger->info(__PACKAGE__, "[ENG:Reporter] * Updating current state for 'legacy':'$lth'");
            $current_state->{'last_ts_legacy'} = $lth;
        }
    }
    return $current_state;
}

=pod

=back

=cut

1;
