package Whois::Remote;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Whois::Remote - Simple whois database constructed by parsing whois dump file

=head1 SYNOPSIS

 use Whois::Remote;

 # Instantinate the whois database
 my $wi_remote = Whois::Remote->new();

 # Find information regarding the given IP address
 my @results = $wi_remote->find('146.102.1.1');

 # Find information regarding the given IP address
 my $abuse = $wi_remote->find_abuse('146.102.1.1');

=head1 DESCRIPTION

Extensive module description

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Whois::Module;
use Net::Whois::IP qw(whoisip_query);
use Value::Convertor;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.6';
    $DEVEL   = 0;
    @ISA     = ('Whois::Module');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item find($) [PUBLIC]

  Usage     : my $result = $instance->find('146.102.1.1');
  Purpose   : Search for given IP address in the inner whois storage
  Arguments : MIXED $ip - IP address [MANDATORY]
  Returns   : ARRAY OF HASH REFERENCES containing network structures
  Throws    : Dies, if not invoked on object instance

=cut

sub find($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($ip,) = @_;

    unless ($self->{CACHE}->{$ip}) {
        my @tmp = whoisip_query($ip);
        my @result;

        foreach my $t (@tmp) {
            if ($t->{'abuse-mailbox'}) {
                $t->{'inetnum'} =~ s/ //g;
                my $r = {};
                my ($min, $max, $type) = $self->_parse_mmt($t->{'inetnum'});

                $r->{'resolved_abuses'} = [$t->{'abuse-mailbox'}];
                $r->{'remarks'}         = $t->{'remarks'};
                $r->{'route'}           = $t->{'route'};
                $r->{'network'}         = $t->{'inetnum'};
                $r->{'netname'}         = $t->{'netname'};
                $r->{'source'}          = $t->{'source'};
                $r->{'description'}     = $t->{'descr'};
                $r->{'first'}           = $min;
                $r->{'last'}            = $max;
                $r->{'type'}            = $type;

                if ($type eq 'ipv4') {
                    $r->{'ip4_start'} = Value::Convertor->int_to_ipv4str($min);
                    $r->{'ip4_end'}   = Value::Convertor->int_to_ipv4str($max);
                }
                push(@result, $r);
            }
        }

        $self->{CACHE}->{$ip} = [@result];
    }
    return @{$self->{CACHE}->{$ip}};
}

=item rfind($) [PUBLIC]

  Usage     : my $result = $instance->rfind('email@abuse.domain');
  Purpose   : Search for given email address in the inner whois storage
  Arguments : MIXED $email - email address [MANDATORY]
  Returns   : ARRAY OF HASH REFERENCES containing network structures
  Throws    : Dies, if not invoked on object instance

=cut

sub rfind($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($email,) = @_;

    return ();
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init($) [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Whois::Negistry instance
#  Arguments : STRING $whois_file - name of the file containing whois dump
#              BOOL $force - parse the whois dump file, even if it is too old
#  Returns   : Whois::Negistry reference
#  Throws    : Dies, if not invoked on object instance
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Result cache
    $self->{CACHE} = {};

    return $self;
}

=pod

=back

=cut

1;
